package jsv.table;

import aos.jack.util.cursor.Action;

/*
 * The JigController class provides access to jig operations (change contents,
 * advance, retract and rotate). The rotate operation is provided by the
 * VirtualController class.
 */
public class JigController extends VirtualController { 

    // jig states
    public static final int STATE0 = 0;     // empty
    public static final int STATE1 = 1;     // A
    public static final int STATE2 = 2;     // A+B
    public static final int STATE3 = 3;     // AB
    public static final int STATE4 = 4;     // C
    public static final int STATE5 = 5;     // C+AB
    public static final int STATE6 = 6;     // CAB

    String[] contentStates = { "0","1","2","3","4","5","6" };

    // slide movements
    public static final int XINCREASING  = 1;   
    public static final int XDECREASING  = -1;  
    public static final int YINCREASING  = 2;   
    public static final int YDECREASING  = -2;  

    // state variables
    String contents;     // indicates the contents of the jig
    int xOffset;    // distance from home for an x-slide
    int yOffset;    // distance from home for a y-slide
    
    int length;     // slide length
    int dt;     // time for a step (msecs)

    public JigController(double da, int dat,int l, int st)
    {
    super(0.0,0.0,0.0,da,dat,0.0,0);
    contents = "0";
    xOffset = 0;
    yOffset = 0;
    length = l;
    dt = st;
    }

// state variable access

    public String getContents() 
    {
        return contents;
    }

    public int getXOffset() 
    {
        return xOffset;
    }

    public int getYOffset() 
    {
        return yOffset;
    }

    public void setContents(int s)
    {
        contents = contentStates[s];
    }

    public void setOffsets( int dx, int dy)
    {
        xOffset = dx;
        yOffset = dy;
    }

// slide operations (advance, retract)

    public Action advance(int d)
    {
    int p1 = (d < 0) ? -length : length; 
    return new SlideAction(0,p1,d);
    }

    public Action retract(int d)
    {
    int p0 = (d > 0) ? -length : length; 
    return new SlideAction(p0,0,d);
    }

    public class SlideAction extends Action {

    int p0;     // starting offset
    int p1;     // finishing offset
    int direction;  // direction of sliding

        SlideAction(int p0,int p1,int d)
    {
            this.p0 = p0;
        this.p1 = p1;
        direction = d;
    }

    protected void action()
    {
        switch(direction)
        {
            case XINCREASING:
                increasex ( p0,p1 );
            break;
        case YINCREASING:
                increasey ( p0,p1 );
            break;
            case XDECREASING:
                decreasex (p0,p1);
            break;
            case YDECREASING:
                decreasey( p0,p1 );
            break;
        default:
            System.err.println("invalid direction");
        }
    }
    }

    void increasex( int p0, int p1 )
    {
    for ( int i = p0; i < p1; i++ )
    {
        xOffset = i+1; 
        delay(dt);
    }
    xOffset = p1;
    }

    void increasey( int p0, int p1 )
    {
    for ( int i = p0; i < p1; i++ )
    {
        yOffset = i+1; 
        delay(dt);
    }
    yOffset = p1;
    }

    void decreasex( int p0, int p1 )
    {
    for ( int i = p0; i > p1; i-- )
    {
        xOffset = i-1; 
        delay(dt);
    }
    xOffset = p1;
    }

    void decreasey( int p0, int p1 )
    {
    for ( int i = p0; i > p1; i-- )
    {
        yOffset = i-1; 
        delay(dt);
    }
    yOffset = 0;
    }
}
